//--------------------------------------------------------------------
//	
//	Slider.cpp
//
//	Written by: Robert Polic
//  Revised: 26-Nov-96 marc
//	
//	Copyright 1995, 1996 Be, Inc. All Rights Reserved.
//	
//--------------------------------------------------------------------

#include "slider.h"

#include <Window.h>

extern char back_bits[];
extern char thumb_bits[];

const float SliderView::fMin = 0.0;
const float SliderView::fMax = 1.0;

const int SliderView::fBackWidth = BACK_WIDTH;
const int SliderView::fBackHeight = BACK_HEIGHT;
const int SliderView::fThumbWidth = THUMB_WIDTH;
const int SliderView::fThumbHeight = THUMB_HEIGHT;
const BRect SliderView::fBackRect (0, 0, BACK_WIDTH, BACK_HEIGHT);
const BRect SliderView::fThumbRect (0, 0, THUMB_WIDTH, THUMB_HEIGHT);

inline float
SliderView::ThumbTop ()
{
  return fBackHeight - 2 - 100*fValue - fThumbHeight;
}

SliderView::SliderView (BPoint where, char* name, float value,
						BLooper* target, long ID)
  : BView (BRect (where, where + BPoint (fBackWidth, fBackHeight)), name,
		   B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_NAVIGABLE),
		fID (ID), fValue (value), fTarget(target)
{
  fSliderBack = new BBitmap (fBackRect, B_COLOR_8_BIT);
  fSliderBack->SetBits (back_bits, fSliderBack->BitsLength(), 0, B_COLOR_8_BIT);

  fThumb = new BBitmap (fThumbRect, B_COLOR_8_BIT, TRUE);
  fThumb->SetBits (thumb_bits, fThumb->BitsLength(), 0, B_COLOR_8_BIT);

  fOffScreen = new BBitmap (fBackRect, B_COLOR_8_BIT, TRUE);
  fOffView = new BView (fBackRect, "slider", B_FOLLOW_ALL, B_WILL_DRAW);
  fOffScreen->Lock ();
  fOffScreen->AddChild (fOffView);
  fOffScreen->Unlock ();

  fXOffset = 0;
}

SliderView::~SliderView ()
{
  delete fSliderBack;
  delete fThumb;
  delete fOffScreen;	/* magically deletes fOffView! */
}

void
SliderView::KeyDown(const char* bytes, int32 numBytes)
{
  if (numBytes == 1)
	if (bytes[0] == B_UP_ARROW)
	  SetValue(fValue + 0.05);
	else if (bytes[0] == B_DOWN_ARROW)
	  SetValue(fValue - 0.05);

  inherited::KeyDown(bytes, numBytes);
}

void
SliderView::MakeFocus(bool state)
{
	if (state == IsFocus())
		return;

	BView::MakeFocus(state);

	if (Window()) {
		Draw(Bounds());
	}
}

void
SliderView::Draw (BRect)
{
  BPoint thumb ((fBackWidth - fThumbWidth) / 2, ThumbTop ());
  fOffScreen->Lock();
  fOffView->DrawBitmap(fSliderBack, BPoint(0, 0));
  fOffView->DrawBitmap(fThumb, thumb);
  if (IsFocus()) {
	BRect highlight (fThumbRect);
	highlight.OffsetTo(thumb);
	highlight.InsetBy(-2, -2);
  	fOffView->StrokeRect(highlight);
  }
  fOffView->Sync();
  DrawBitmap(fOffScreen, BPoint(fXOffset, 0));
  fOffScreen->Unlock();
}

void
SliderView::AttachedToWindow ()
{
  SetViewColor (B_TRANSPARENT_32_BIT);
}

float
SliderView::Value ()
{
  return (fValue);
}

void
SliderView::SetValue (float value)
{
  if (value < fMin)
	value = fMin;
  if (value > fMax)
	value = fMax;

  if (value != fValue) {
	if (fTarget) {
	  BMessage* mes = new BMessage(fID);
	  fValue = value;
	  mes->AddFloat("value", value);
	  fTarget->PostMessage(mes);
	}
	Draw (BRect());
  }
}

void
SliderView::MouseDown (BPoint click)
{
  BPoint where;
  ulong buttons;
  GetMouse (&where, &buttons);

  BRect rect = fThumbRect;
  rect.OffsetBy (fXOffset + 6, ThumbTop ());
  if (!rect.Contains (click))
	SetValue ((fBackHeight - 13 - click.y) / 100.0);
  else
	SetValue (fValue);

  float old_value = fValue;
  while (buttons) {
	GetMouse (&where, &buttons);
	SetValue (old_value + (click.y - where.y) / 100.0);
	BWindow* win = Window();
	win->Unlock();
	snooze (20000);
	if (!win->Lock())
	  break;
  }
}


char back_bits[] = {
0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,
0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xa,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0xa,0x0,0x0,
0x0,0x0,0x0,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0xa,
0x0,0x0,0x4,0x5,0x4,0xff,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,0xa,0xa,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0xa,0xa,0xa,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0xff,0xff,
0xff,0xff,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0xff,0xff,0xff,0xff,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0xa,0xa,0xa,0xa,0xa,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0xff,0xff,0xff,0xff,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xff,0xff,0xff,0xff,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,0xa,0xa,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0xff,0xff,0xff,0xff,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xff,0xff,
0xff,0xff,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,0xa,0xa,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0xa,0xa,0xa,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0xff,0xff,
0xff,0xff,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0xff,0xff,0xff,0xff,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,
0xa,0xa,0xa,0xa,0xa,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x5,0x4,
0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0xa,0xa,0xa,0x13,0xff,0xff,0x1e,0x1e,
0x1e,0xa,0x13,0xff,0xff,0xff,0xff,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,
0x5,0x4,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0xff,0xff,0xff,0xff,0x13,0xff,0xff,
0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,
0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,
0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,
0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,
0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,
0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,0x4,0x4,
0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,0xff,0x1e,
0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,0x0,0x0,
0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x13,0xff,
0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0xa,
0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x17,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0xa,0x0,0x0,0x4,0x4,0x5,0xff,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,
0x1e,0x1e,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xa,0x13,0x13,0x13,0x13,0x13,0x13,
0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,
0x13,0x13,0x13,0x13,0x13,0xff,0xff,0x1e,0x1e,0x1e,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x1e,0x1e,0x1e,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x1e,0x1e,0x1e};

char thumb_bits[] = {
0x15,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x1e,0x1e,0,0,
0x0d,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x00,0x0f,0,0,
0x0d,0x15,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x0f,0,0,
0x0d,0x15,0xff,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x13,0xff,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x11,0xff,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x11,0xff,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0f,0xff,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0f,0x1e,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0c,0x1e,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0c,0x1b,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0d,0x1b,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x0a,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0d,0x1b,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0d,0x1e,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0e,0x1e,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0f,0x1e,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x13,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x0f,0xff,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x11,0xff,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x15,0x17,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x11,0xff,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x1b,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x15,0xff,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x17,0x1b,0xff,0xff,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x15,0xff,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x08,0x08,0x00,0x0f,0,0,
0x0d,0x15,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,0x0f,0,0,
0x1e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0,0,
0x1e,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x1e,0,0};
