//************************************************************************************
//	File:			KeyMapMaker.cpp
//
//	Description:	Be keyboard mapping tables for mapping from raw key code
//					to ASCII values.
//
//	Written by:		Robert Polic
//
//	Copyright 1994, Be Incorporated, All Rights Reserved.
//
//************************************************************************************

#include "stdio.h"
#ifndef INTERFACE_DEFS_H
#include "InterfaceDefs.h"
#endif


//------------------------------------------------------------------------------------------------------------------------------------
//	Raw key numbering for 101 keyboard...
//                                                                                        [sys]       [brk]
//                                                                                         0x7e        0x7f
// [esc]       [ f1] [ f2] [ f3] [ f4] [ f5] [ f6] [ f7] [ f8] [ f9] [f10] [f11] [f12]    [prn] [scr] [pau]
//  0x01        0x02  0x03  0x04  0x05  0x06  0x07  0x08  0x09  0x0a  0x0b  0x0c  0x0d     0x0e  0x0f  0x10     K E Y P A D   K E Y S
//
// [ ` ] [ 1 ] [ 2 ] [ 3 ] [ 4 ] [ 5 ] [ 6 ] [ 7 ] [ 8 ] [ 9 ] [ 0 ] [ - ] [ = ] [bck]    [ins] [hme] [pup]    [num] [ / ] [ * ] [ - ]
//  0x11  0x12  0x13  0x14  0x15  0x16  0x17  0x18  0x19  0x1a  0x1b  0x1c  0x1d  0x1e     0x1f  0x20  0x21     0x22  0x23  0x24  0x25
//
// [tab] [ q ] [ w ] [ e ] [ r ] [ t ] [ y ] [ u ] [ i ] [ o ] [ p ] [ [ ] [ ] ] [ \ ]    [del] [end] [pdn]    [ 7 ] [ 8 ] [ 9 ] [ + ]
//  0x26  0x27  0x28  0x29  0x2a  0x2b  0x2c  0x2d  0x2e  0x2f  0x30  0x31  0x32  0x33     0x34  0x35  0x36     0x37  0x38  0x39  0x3a
//
// [cap] [ a ] [ s ] [ d ] [ f ] [ g ] [ h ] [ j ] [ k ] [ l ] [ ; ] [ ' ] [  enter  ]                         [ 4 ] [ 5 ] [ 6 ]
//  0x3b  0x3c  0x3d  0x3e  0x3f  0x40  0x41  0x42  0x43  0x44  0x45  0x46     0x47                             0x48  0x49  0x4a
//
// [shift]     [ z ] [ x ] [ c ] [ v ] [ b ] [ n ] [ m ] [ , ] [ . ] [ / ]     [shift]          [ up]          [ 1 ] [ 2 ] [ 3 ] [ent]
//   0x4b       0x4c  0x4d  0x4e  0x4f  0x50  0x51  0x52  0x53  0x54  0x55       0x56            0x57           0x58  0x59  0x5a  0x5b
//
// [ctr]             [cmd]             [  space  ]             [cmd]             [ctr]    [lft] [dwn] [rgt]    [ 0 ] [ . ]
//  0x5c              0x5d                 0x5e                 0x5f              0x60     0x61  0x62  0x63     0x64  0x65
//
//	NOTE: On a Microsoft Natural Keyboard:
//			left option  = 0x66
//			right option = 0x67
//			menu key     = 0x68
//------------------------------------------------------------------------------------------------------------------------------------


unsigned long mappingTables[] = {
/* version */				2,

//------------------------------------------------------------------------------------
//	The modifier tables:
//------------------------------------------------------------------------------------
//
/* caps lock key: */		59,
/* scroll lock key: */		15,
/* num lock key: */			34,
/* left shift key: */		75,
/* right shift key: */		86,
/* left command key: */		93,
/* right command key: */	95,
/* left control key: */		92,
/* right control key: */	0,
/* left option key: */		102,
/* right option key: */		96,
/* menu key: */				104,
/* lock settings: */		0,


//------------------------------------------------------------------------------------
//	The mapping tables (in order of priority):
//
//	NOTE: When mapping keypad keys when NumLock is enabled, the shift version
//		  of a table will use the non-shift version and the non-shift version
//		  will use the shift version (i.e. table 2 uses table 3, table 3 uses
//		  table 2).
//

// Control table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16, -1, -1,  0, -1, -1, -1, 30, -1, -1, -1, -1, 31, -1,127,  5,
/* 2 */  1, 11, -1,'/','*','-',  9, 17, 23,  5, 18, 20, 25, 21,  9, 15,
/* 3 */ 16, 27, 29, 28,127,  4, 12,  1, 30, 11,'+', -1,  1, 19,  4,  6,
/* 4 */  7,  8, 10, 11, 12, -1, -1, 13, 28, -1, 29, -1, 26, 24,  3, 22,
/* 5 */  2, 14, 13, -1, -1, -1, -1, 30,  4, 31, 12, 13, -1, -1,  0, -1,
/* 6 */ -1, 28, 31, 29,  5,127, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Option-Caps-Shift table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`', -1, -1, -1, -1, -1,246,164,161,187,188,209,177,  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,207, -1,171, -1,224, -1, -1, -1,191,
/* 3 */185,210,211, -1,127,  4, 12,'7','8','9','+', -1,140, -1, -1, -1,
/* 4 */ -1, -1, -1, -1,190,172,247, 13,'4','5','6', -1, -1, -1,141, -1,
/* 5 */ -1,150, -1,178,179,192, -1, 30,'1','2','3', 13, -2, -1,202, -1,
/* 6 */ -1, 28, 31, 29,'0','.', -1, -1, -1,'|', -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Option-Caps table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`',193,170,163,162,176,246,166,165,199,200,208,173,  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,206,183,171,168,160,180, -1, -1,175,
/* 3 */184,212,213,194,127,  4, 12,  1, 30, 11,'+', -1,203,186,182,196,
/* 4 */169, -1,198,215,174,172,247, 13, 28, -1, 29, -1,189,197,130,195,
/* 5 */167,132,181, -1,201,214, -1, 30,  4, 31, 12, 13, -1, -1,202, -1,
/* 6 */ -1, 28, 31, 29,  5,127, -1, -1, -1, 92, -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Option-Shift table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`', -1, -1, -1, -1,228,246,164,161,187,188,209,177,  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,206, -1,171, -1,224, -1, -1, -1,175,
/* 3 */184,210,211, -1,127,  4, 12,'7','8','9','+', -1,129, -1, -1, -1,
/* 4 */ -1, -1, -1, -1,174,172,247, 13,'4','5','6', -1, -1, -1,130, -1,
/* 5 */ -1,132, -1,178,179,192, -1, 30,'1','2','3', 13, -1, -1,202, -1,
/* 6 */ -1, 28, 31, 29,'0','.', -1, -1, -1,'|', -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Option table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`',193,170,163,162,176,246,166,165,199,200,208,173,  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,207,183,171,168,160,180, -1,222,191,
/* 3 */185,212,213,194,127,  4, 12,  1, 30, 11,'+', -1,140,186,182,196,
/* 4 */169, -1,198,215,190,172,247, 13, 28, -1, 29, -1,189,197,141,195,
/* 5 */167,150,181, -1,201,214, -1, 30,  4, 31, 12, 13, -1, -1,202, -1,
/* 6 */ -1, 28, 31, 29,  5,127, -1, -1, -1, 92, -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Caps-Shift table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'~','!','@','#','$','%','^','&','*','(',')','_','+',  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,'q','w','e','r','t','y','u','i','o',
/* 3 */'p','{','}','|',127,  4, 12,'7','8','9','+', -1,'a','s','d','f',
/* 4 */'g','h','j','k','l',':','"', 13,'4','5','6', -1,'z','x','c','v',
/* 5 */'b','n','m','<','>','?', -1, 30,'1','2','3', 13, -1, -1,' ', -1,
/* 6 */ -1, 28, 31, 29,'0','.', -1, -1, -1,'|', -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Caps table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`','1','2','3','4','5','6','7','8','9','0','-','=',  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,'Q','W','E','R','T','Y','U','I','O',
/* 3 */'P','[',']', 92,127,  4, 12,  1, 30, 11,'+', -1,'A','S','D','F',
/* 4 */'G','H','J','K','L',';', 39, 13, 28, -1, 29, -1,'Z','X','C','V',
/* 5 */'B','N','M',',','.','/', -1, 30,  4, 31, 12, 13, -1, -1,' ', -1,
/* 6 */ -1, 28, 31, 29,  5,127, -1, -1, -1, 92, -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Shift table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'~','!','@','#','$','%','^','&','*','(',')','_','+',  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,'Q','W','E','R','T','Y','U','I','O',
/* 3 */'P','{','}','|',127,  4, 12,'7','8','9','+', -1,'A','S','D','F',
/* 4 */'G','H','J','K','L',':','"', 13,'4','5','6', -1,'Z','X','C','V',
/* 5 */'B','N','M','<','>','?', -1, 30,'1','2','3', 13, -1, -1,' ', -1,
/* 6 */ -1, 28, 31, 29,'0','.', -1, -1, -1,'|', -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// No mods table...
/*      -0- -1- -2- -3- -4- -5- -6- -7- -8- -9- -a- -b- -c- -d- -e- -f-   */

/* 0 */ -1, 27, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
/* 1 */ 16,'`','1','2','3','4','5','6','7','8','9','0','-','=',  8,  5,
/* 2 */  1, 11, -1,'/','*','-',  9,'q','w','e','r','t','y','u','i','o',
/* 3 */'p','[',']', 92,127,  4, 12,  1, 30, 11,'+', -1,'a','s','d','f',
/* 4 */'g','h','j','k','l',';', 39, 13, 28, -1, 29, -1,'z','x','c','v',
/* 5 */'b','n','m',',','.','/', -1, 30,  4, 31, 12, 13, -1, -1,' ', -1,
/* 6 */ -1, 28, 31, 29,  5,127, -1, -1, -1, 92, -1, -1, -1, -1, -1, -1,
/* 7 */ -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,200,202,

// Acute dead key table...
' ', 171,
'A', 231,
'E', 131,
'I', 234,
'O', 238,
'U', 242,
'a', 135,
'e', 142,
'i', 146,
'o', 151,
'u', 156,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,

// Grave dead key table...
' ', '`',
'A', 203,
'E', 233,
'I', 237,
'O', 241,
'U', 244,
'a', 136,
'e', 143,
'i', 147,
'o', 152,
'u', 157,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,

// Circumflex dead key table...
' ', 246,
'A', 229,
'E', 230,
'I', 235,
'O', 239,
'U', 243,
'a', 137,
'e', 144,
'i', 148,
'o', 153,
'u', 158,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,

// Diaeresis dead key table...
' ', 172,
'A', 128,
'E', 232,
'I', 236,
'O', 133,
'U', 134,
'Y', 217,
'a', 138,
'e', 145,
'i', 149,
'o', 154,
'u', 159,
'y', 216,
  0,   0,
  0,   0,
  0,   0,

// Tilde dead key table...
' ', 247,
'A', 204,
'N', 132,
'O', 205,
'a', 139,
'n', 150,
'o', 155,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,
  0,   0,

// Acute table mask...
B_OPTION_CAPS_SHIFT_TABLE | B_OPTION_CAPS_TABLE |
B_OPTION_SHIFT_TABLE | B_OPTION_TABLE,

// Grave table mask...
B_OPTION_CAPS_SHIFT_TABLE | B_OPTION_CAPS_TABLE |
B_OPTION_SHIFT_TABLE | B_OPTION_TABLE,

// Circumflex table mask...
B_OPTION_CAPS_SHIFT_TABLE | B_OPTION_CAPS_TABLE |
B_OPTION_SHIFT_TABLE | B_OPTION_TABLE,

// Diaeresis table mask...
B_OPTION_CAPS_SHIFT_TABLE | B_OPTION_CAPS_TABLE |
B_OPTION_SHIFT_TABLE | B_OPTION_TABLE,

// Tilde dead key table...
B_OPTION_CAPS_SHIFT_TABLE | B_OPTION_CAPS_TABLE |
B_OPTION_SHIFT_TABLE | B_OPTION_TABLE
};


//------------------------------------------------------------------------------------
//	Compile and run to make the Key_map file...
//

main()
{
	FILE*	mapFile;

	printf("Creating Key_map file...");
	mapFile = fopen("Key_map", "w");
	fwrite(mappingTables, 1, sizeof(mappingTables), mapFile);
	fclose(mapFile);
	printf("done.\n");
}
