# DeviceSpecTemplate.mk -- template for a device spec
# Copyright (c) 2000-2001; Be, Inc. All Rights Reserved.
#
# Note: This is for .spec files living in ${BUILDHOME}/config/spec.
#
# Input Parameters: (they have to set before the template inclusion) 
#  *DEVICE_NAME:=	the name of the device
#	PARENT_DEVICE:=	the name of the device this was based upon
#	CPU_TYPE	:=	cpu-type
#   ALL_ELF :=		files required for any package with this device
#   ALL_PLAIN :=	files required for any package with this device
#   ALL_DIR :=		files required for any package with this device
#   ALL_LINK :=		files required for any package with this device
#	ALL_MFLAGS :=	mflags required for any package with this device
#	ALL_DFLAGS :=	dflags required for any package with this device
#	ALL_MVARS :=	mvars required for any package with this device
#   NOT_ALL_ELF :=	files not installed for any package with this device
#					(that otherwise would be installed)
#   NOT_ALL_PLAIN :=files not installed for any package with this device
#					(that otherwise would be installed)
#   NOT_ALL_DIR :=	files not installed for any package with this device
#					(that otherwise would be installed)
#   NOT_ALL_LINK :=	files not installed for any package with this device
#					(that otherwise would be installed)
#   NOT_ALL_GRAPHICS :=	files not installed for any package with this device
#					(that otherwise would be installed)
#   NOT_ALL_LBX :=	files not installed for any package with this device
#					(that otherwise would be installed)
#	NOT_ALL_MFLAGS := mflags not installed for any package with this device
#					(that otherwise would be installed)
#	NOT_ALL_DFLAGS := dflags not installed for any package with this device
#					(that otherwise would be installed)
#   BEIA_ELF := 	files required for any BEIA-based package with this device
#   BEIA_PLAIN := 	files required for any BEIA-based package with this device
#   BEIA_DIR := 	files required for any BEIA-based package with this device
#   BEIA_LINK := 	files required for any BEIA-based package with this device
#	BEIA_MFLAGS :=	mflags required for any BEIA-based package with this device
#	BEIA_DFLAGS :=	dflags required for any BEIA-based package with this device
#	BEIA_MVARS :=	mvars required for any BEIA-based package with this device
#   NOT_BEIA_ELF := 	like NOT_ALL equilivent, only for BEIA-based packages
#   NOT_BEIA_PLAIN := 	like NOT_ALL equilivent, only for BEIA-based packages
#   NOT_BEIA_DIR := 	like NOT_ALL equilivent, only for BEIA-based packages
#   NOT_BEIA_LINK := 	like NOT_ALL equilivent, only for BEIA-based packages
#	NOT_BEIA_MFLAGS :=	like NOT_ALL equilivent, only for BEIA-based packages
#	NOT_BEIA_DFLAGS :=	like NOT_ALL equilivent, only for BEIA-based packages
#	NOT_IARELEASE_ELF := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#	NOT_IARELEASE_PLAIN := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#	NOT_IARELEASE_DIR := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#	NOT_IARELEASE_LINK := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#	NOT_IARELEASE_MFLAGS := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#	NOT_IARELEASE_DFLAGS := NOT_BEIA_* removes from all IA builds. NOT_IARELEASE_*
#						removes from just the release build
#   IATOOLS_ELF := 	files required for any BEIA_TOOLS-based package with this device
#   IATOOLS_PLAIN := files required for any BEIA_TOOLS-based package with this device
#   IATOOLS_DIR := 	files required for any BEIA_TOOLS-based package with this device
#   IATOOLS_LINK := files required for any BEIA_TOOLS-based package with this device
#	IATOOLS_MFLAGS := mflags required for any BEIA_TOOLS-based package with this device
#	IATOOLS_DFLAGS := dflags required for any BEIA_TOOLS-based package with this device
#	IATOOLS_MVARS := mvars required for any BEIA_TOOLS-based package with this device
#   NOT_IATOOLS_ELF := 	like NOT_ALL equilivent...
#   NOT_IATOOLS_PLAIN := 	like NOT_ALL equilivent...
#   NOT_IATOOLS_DIR := 	like NOT_ALL equilivent...
#   NOT_IATOOLS_LINK := 	like NOT_ALL equilivent...
#	NOT_IATOOLS_MFLAGS :=	like NOT_ALL equilivent...
#	NOT_IATOOLS_DFLAGS :=	like NOT_ALL equilivent...
#   INSTALLER_ELF := 	files required for any BEIA_INSTALLER build
#   INSTALLER_PLAIN := files required for any BEIA_INSTALLER build
#   INSTALLER_DIR := 	files required for any BEIA_INSTALLER build
#   INSTALLER_LINK := files required for any BEIA_INSTALLER build
#	INSTALLER_MFLAGS := mflags required for any BEIA_INSTALLER build
#	INSTALLER_DFLAGS := dflags required for any BEIA_INSTALLER build
#	INSTALLER_MVARS := mvars required for any BEIA_INSTALLER build
#   NOT_INSTALLER_ELF := 	like NOT_ALL equilivent...
#   NOT_INSTALLER_PLAIN := 	like NOT_ALL equilivent...
#   NOT_INSTALLER_DIR := 	like NOT_ALL equilivent...
#   NOT_INSTALLER_LINK := 	like NOT_ALL equilivent...
#	NOT_INSTALLER_MFLAGS :=	like NOT_ALL equilivent...
#	NOT_INSTALLER_DFLAGS :=	like NOT_ALL equilivent...
#   BEOS_WAGNER_ELF := 	files required for any BEOS-based package with this device
#   BEOS_WAGNER_PLAIN := 	files required for any BEOS-based package with this device
#   BEOS_WAGNER_DIR := 	files required for any BEOS-based package with this device
#   BEOS_WAGNER_LINK := 	files required for any BEOS-based package with this device
#	BEOS_WAGNER_MFLAGS :=	mflags required for any BEOS-based package with this device
#	BEOS_WAGNER_DFLAGS :=	dflags required for any BEOS-based package with this device
#	BEOS_WAGNER_MVARS :=	mvars required for any BEOS-based package with this device
#   NOT_BEOS_WAGNER_ELF := 	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#   NOT_BEOS_WAGNER_PLAIN := 	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#   NOT_BEOS_WAGNER_DIR := 	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#   NOT_BEOS_WAGNER_LINK := 	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#	NOT_BEOS_WAGNER_MFLAGS :=	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#	NOT_BEOS_WAGNER_DFLAGS :=	like NOT_ALL equilivent, only for BEOS_WAGNER-based packages
#   BEOS_ELF := 	files required for any BEOS-based package with this device
#   BEOS_PLAIN := 	files required for any BEOS-based package with this device
#   BEOS_DIR := 	files required for any BEOS-based package with this device
#   BEOS_LINK := 	files required for any BEOS-based package with this device
#	BEOS_MFLAGS :=	mflags required for any BEOS-based package with this device
#	BEOS_DFLAGS :=	dflags required for any BEOS-based package with this device
#	BEOS_MVARS :=	mvars required for any BEOS-based package with this device
#   NOT_BEOS_ELF := 	like NOT_ALL equilivent, only for BEOS-based packages
#   NOT_BEOS_PLAIN := 	like NOT_ALL equilivent, only for BEOS-based packages
#   NOT_BEOS_DIR := 	like NOT_ALL equilivent, only for BEOS-based packages
#   NOT_BEOS_LINK := 	like NOT_ALL equilivent, only for BEOS-based packages
#	NOT_BEOS_MFLAGS :=	like NOT_ALL equilivent, only for BEOS-based packages
#	NOT_BEOS_DFLAGS :=	like NOT_ALL equilivent, only for BEOS-based packages
#   OSDEBUG_ELF := 	files required for any BEOS_DEBUG-based package with this device
#   OSDEBUG_PLAIN := 	files required for any BEOS_DEBUG-based package with this device
#   OSDEBUG_DIR := 	files required for any BEOS_DEBUG-based package with this device
#   OSDEBUG_LINK := 	files required for any BEOS_DEBUG-based package with this device
#	OSDEBUG_MFLAGS :=	mflags required for any BEOS_DEBUG-based package with this device
#	OSDEBUG_DFLAGS :=	dflags required for any BEOS_DEBUG-based package with this device
#	OSDEBUG_MVARS :=	mvars required for any BEOS_DEBUG-based package with this device
#   NOT_OSDEBUG_ELF := 	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages
#   NOT_OSDEBUG_PLAIN := 	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages
#   NOT_OSDEBUG_DIR := 	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages
#   NOT_OSDEBUG_LINK := 	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages
#	NOT_OSDEBUG_MFLAGS :=	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages
#	NOT_OSDEBUG_DFLAGS :=	like NOT_ALL equilivent, only for BEOS_DEBUG-based packages

#
# Output Parameters: (they become valid after the template inclusion, and are
# invalidated by any other template inclusion)
#
#	$(DEVICE_NAME)-based variables
#	All input variables are cleared.
#   The only input variables carried over to *_IN variables are DEVICE_NAME
#   and PARENT_DEVICE
#
# Todo:
#
#	There are still sections that can't be customized.
#	They should be added as needed.
#
# Bugs:
#
#   Because of the sheer number of input variables, most don't have _IN equilivents.
#
#   This totally doesn't take in to account the LBX or GRAPHICS _PAK variables.
#   There are other issues here as well, as even if this did take into account
#   LBX and GRAPHICS _PAK variables, when you removed something from a GRAPHICS
#   pak it would remain inside the LBX file.
#
# Notes:
#	The PARENT_DEVICE *must* be defined before any of the
#	input variables to this are set.
#
#   The heirarchy of packages:
#   + BEIA_RELEASE
#     + BEIA_TOOLS
#       + BEOS_BEIA_DEVELOP (*)
#       + BEIA_DEBUG
#         + BEIA_DEVELOP
#         + BEIA_DEBUG_SMALL
#       + BEIA_INSTALLER
#   + BEOS_RELEASE
#     + BEOS_MINIMAL (*)
#     + BEOS_DEBUG
#       + BEOS_MINIMAL_DEBUG (*)
#     + BEOS_FUNCTIONAL
#     + BEOS_DEVELOP
#   + BEOS_WAGNER
#     + BEOS_WAGNER_NO_INTERFACE
#   + BEIA_MIRROR_ENVIRONMENT
#
# (*) marks special cases that inherit MFLAGS, DFLAGS, and MVARS
#     but don't inherit any files.
#
############################################################

# This file is actually nothing more than a giant list
# of definitions.

# Strip the variables as we plop them in their output locations
DEVICE_NAME_IN := $(strip $(DEVICE_NAME))
PARENT_DEVICE_IN :=	$(strip $(PARENT_DEVICE))

$(DEVICE_NAME)_CPU_TYPE_SPEC:=$(strip \
	$(if $(strip $(CPU_TYPE)),\
		$(CPU_TYPE), \
		$(if $(PARENT_DEVICE_IN), \
			$($(PARENT_DEVICE_IN)_CPU_TYPE_SPEC), \
			$($(CPU_TYPE_PENTIUM_AND_HIGHER_SPEC)) \
		) \
	) \
)

$(DEVICE_NAME_IN)_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_FILES_ELF_SPEC),) \
	$(ALL_ELF) \
#

$(DEVICE_NAME_IN)_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_FILES_PLAIN_SPEC),) \
	$(ALL_PLAIN) \
#

$(DEVICE_NAME_IN)_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_FILES_LINK_SPEC),) \
	$(ALL_LINK) \
#

$(DEVICE_NAME_IN)_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_FILES_DIR_SPEC)),) \
	$(ALL_DIR) \
#

$(DEVICE_NAME_IN)_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $(filter-out $($($(PARENT_DEVICE_NAME)_CPU_TYPE_SPEC)_MPAK),$($(PARENT_DEVICE_IN)_MFLAGS_SPEC)),) \
	$($($(DEVICE_NAME)_CPU_TYPE_SPEC)_MPAK) \
	$(ALL_MFLAGS) \
#

$(DEVICE_NAME_IN)_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_DFLAGS_SPEC),) \
	$(ALL_DFLAGS) \
#

$(DEVICE_NAME_IN)_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(ALL_MVARS), $($(PARENT_DEVICE_IN)_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_ALL_ELF) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_ALL_PLAIN) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_ALL_LINK) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_ALL_DIR) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_GRAPHICS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_GRAPHICS_SPEC),) \
	$(NOT_ALL_GRAPHICS) \
#

$(DEVICE_NAME_IN)_REMOVE_FILES_LBX_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_FILES_LBX_SPEC),) \
	$(NOT_ALL_LBX) \
#

$(DEVICE_NAME_IN)_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_MFLAGS_SPEC),) \
	$(NOT_ALL_MFLAGS) \
#

$(DEVICE_NAME_IN)_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_REMOVE_DFLAGS_SPEC),) \
	$(NOT_ALL_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IARELEASE_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IARELEASE_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IARELEASE_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IARELEASE_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IARELEASE_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_RELEASE_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_RELEASE_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IARELEASE_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
	$(IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
	$(IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
	$(IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
	$(IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_TOOLS_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_TOOLS_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
	$(IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
	$(IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
	$(IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
	$(IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
	$(IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
	$(IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
	$(IATOOLS_LINK) \

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
	$(IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEBUG_SMALL_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEBUG_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
	$(IATOOLS_ELF) \
	$(INSTALLER_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
	$(IATOOLS_PLAIN) \
	$(INSTALLER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
	$(IATOOLS_LINK) \
	$(INSTALLER_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
	$(IATOOLS_DIR) \
	$(INSTALLER_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
	$(INSTALLER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
	$(INSTALLER_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(INSTALLER_MVARS), $(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_MVARS_SPEC)))) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IATOOLS_ELF) \
	$(NOT_INSTALLER_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IATOOLS_PLAIN) \
	$(NOT_INSTALLER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IATOOLS_LINK) \
	$(NOT_INSTALLER_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IATOOLS_DIR) \
	$(NOT_INSTALLER_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
	$(NOT_INSTALLER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_INSTALLER_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_INSTALLER_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
	$(NOT_INSTALLER_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_FILES_ELF_SPEC),) \
	$(BEIA_ELF) \
	$(IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_FILES_PLAIN_SPEC),) \
	$(BEIA_PLAIN) \
	$(IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_FILES_LINK_SPEC),) \
	$(BEIA_LINK) \
	$(IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_FILES_DIR_SPEC)),) \
	$(BEIA_DIR) \
	$(IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEIA_ELF) \
	$(NOT_IATOOLS_ELF) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEIA_PLAIN) \
	$(NOT_IATOOLS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEIA_LINK) \
	$(NOT_IATOOLS_LINK) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEIA_DIR) \
	$(NOT_IATOOLS_DIR) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEIA_DEVELOP_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEIA_DEVELOP_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_FILES_ELF_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_FILES_PLAIN_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_FILES_LINK_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_FILES_DIR_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_MFLAGS_SPEC),) \
	$(BEIA_MFLAGS) \
	$(IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_DFLAGS_SPEC),) \
	$(BEIA_DFLAGS) \
	$(IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(IATOOLS_MVARS), $(call MVAR_REPLACE, $(BEIA_MVARS), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_MVARS_SPEC))) \
#
$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_ELF_SPEC),) \
	$(ALL_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_PLAIN_SPEC),) \
	$(ALL_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_LINK_SPEC),) \
	$(ALL_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_FILES_DIR_SPEC)),) \
	$(ALL_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEIA_MFLAGS) \
	$(NOT_IATOOLS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_BEIA_DEVELOP_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_BEIA_DEVELOP_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEIA_DFLAGS) \
	$(NOT_IATOOLS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_FILES_ELF_SPEC),) \
	$(BEOS_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_FILES_PLAIN_SPEC),) \
	$(BEOS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_FILES_LINK_SPEC),) \
	$(BEOS_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_FILES_DIR_SPEC)),) \
	$(BEOS_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEOS_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_RELEASE_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_RELEASE_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_FILES_ELF_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_FILES_PLAIN_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_FILES_LINK_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_FILES_DIR_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEOS_ELF) \
	$(ALL_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_PLAIN) \
	$(ALL_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_LINK) \
	$(ALL_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_DIR) \
	$(ALL_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_FILES_ELF_SPEC),) \
	$(BEOS_ELF) \
	$(OSDEBUG_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_FILES_PLAIN_SPEC),) \
	$(BEOS_PLAIN) \
	$(OSDEBUG_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_FILES_LINK_SPEC),) \
	$(BEOS_LINK) \
	$(OSDEBUG_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_FILES_DIR_SPEC)),) \
	$(BEOS_DIR) \
	$(OSDEBUG_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
	$(OSDEBUG_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
	$(OSDEBUG_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(OSDEBUG_MVARS), $(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEOS_ELF) \
	$(NOT_OSDEBUG_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_PLAIN) \
	$(NOT_OSDEBUG_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_LINK) \
	$(NOT_OSDEBUG_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_DIR) \
	$(NOT_OSDEBUG_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_MFLAGS) \
	$(NOT_OSDEBUG_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEBUG_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEBUG_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_DFLAGS) \
	$(NOT_OSDEBUG_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_FILES_ELF_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_FILES_PLAIN_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_FILES_LINK_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_FILES_DIR_SPEC := \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
	$(OSDEBUG_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
	$(OSDEBUG_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(OSDEBUG_MVARS), $(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_MVARS_SPEC))) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_ELF_SPEC),) \
	$(NOT_BEOS_ELF) \
	$(NOT_OSDEBUG_ELF) \
	$(ALL_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_PLAIN) \
	$(NOT_OSDEBUG_PLAIN) \
	$(ALL_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_LINK) \
	$(NOT_OSDEBUG_LINK) \
	$(ALL_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_DIR) \
	$(NOT_OSDEBUG_DIR) \
	$(ALL_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_MFLAGS) \
	$(NOT_OSDEBUG_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_MINIMAL_DEBUG_REMOVE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_MINIMAL_DEBUG_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_DFLAGS) \
	$(NOT_OSDEBUG_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_FILES_ELF_SPEC),) \
	$(BEOS_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_FILES_PLAIN_SPEC),) \
	$(BEOS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_FILES_LINK_SPEC),) \
	$(BEOS_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_FILES_DIR_SPEC)),) \
	$(BEOS_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_FUNCTIONAL_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_FUNCTIONAL_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_FILES_ELF_SPEC),) \
	$(BEOS_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_FILES_PLAIN_SPEC),) \
	$(BEOS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_FILES_LINK_SPEC),) \
	$(BEOS_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_FILES_DIR_SPEC)),) \
	$(BEOS_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_TOOLS_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_TOOLS_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_FILES_ELF_SPEC),) \
	$(BEOS_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_FILES_PLAIN_SPEC),) \
	$(BEOS_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_FILES_LINK_SPEC),) \
	$(BEOS_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_FILES_DIR_SPEC)),) \
	$(BEOS_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_MFLAGS_SPEC),) \
	$(BEOS_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_DFLAGS_SPEC),) \
	$(BEOS_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_DEVELOP_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_MVARS), $($(PARENT_DEVICE_IN)_BEOS_DEVELOP_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS__FILES_ELF_SPEC),) \
	$(BEOS_WAGNER_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_PLAIN_SPEC),) \
	$(BEOS_WAGNER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_LINK_SPEC),) \
	$(BEOS_WAGNER_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_DIR_SPEC)),) \
	$(BEOS_WAGNER_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_MFLAGS_SPEC),) \
	$(BEOS_WAGNER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_DFLAGS_SPEC),) \
	$(BEOS_WAGNER_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_WAGNER_MVARS), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS__FILES_REMOVE_ELF_SPEC),) \
	$(NOT_BEOS_WAGNER_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_WAGNER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_REMOVE_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_WAGNER_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_REMOVE_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_WAGNER_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_MFLAGS_REMOVE_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_WAGNER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_DFLAGS_REMOVE_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_WAGNER_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS__FILES_ELF_SPEC),) \
	$(BEOS_WAGNER_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_PLAIN_SPEC),) \
	$(BEOS_WAGNER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_FILES_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_LINK_SPEC),) \
	$(BEOS_WAGNER_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_FILES_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_FILES_DIR_SPEC)),) \
	$(BEOS_WAGNER_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_MFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_MFLAGS_SPEC),) \
	$(BEOS_WAGNER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_DFLAGS_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_DFLAGS_SPEC),) \
	$(BEOS_WAGNER_DFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_MVARS_SPEC := \
	$(call MVAR_REPLACE, $(BEOS_WAGNER_MVARS), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_MVARS_SPEC)) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_REMOVE_FILES_ELF_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS__FILES_REMOVE_ELF_SPEC),) \
	$(NOT_BEOS_WAGNER_ELF) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_REMOVE_FILES_PLAIN_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_PLAIN_SPEC),) \
	$(NOT_BEOS_WAGNER_PLAIN) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_FILES_REMOVE_LINK_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_LINK_SPEC),) \
	$(NOT_BEOS_WAGNER_LINK) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_FILES_REMOVE_DIR_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_FILES_DIR_SPEC)),) \
	$(NOT_BEOS_WAGNER_DIR) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_MFLAGS_REMOVE_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_MFLAGS_SPEC),) \
	$(NOT_BEOS_WAGNER_MFLAGS) \
#

$(DEVICE_NAME_IN)_BEOS_WAGNER_NO_INTERFACE_DFLAGS_REMOVE_SPEC := \
	$(if $(PARENT_DEVICE_IN), $($(PARENT_DEVICE_IN)_BEOS_WAGNER_REMOVE_DFLAGS_SPEC),) \
	$(NOT_BEOS_WAGNER_DFLAGS) \
#

# Clear all input variables

DEVICE_NAME:=	
PARENT_DEVICE:=	
ALL_ELF :=		
ALL_PLAIN :=	
ALL_DIR :=		
ALL_LINK :=		
ALL_MFLAGS :=	
ALL_DFLAGS :=	
ALL_MVARS :=	
NOT_ALL_ELF :=	
NOT_ALL_PLAIN :=
NOT_ALL_DIR :=	
NOT_ALL_LINK :=	
NOT_ALL_GRAPHICS :=	
NOT_ALL_LBX :=	
NOT_ALL_MFLAGS := 
NOT_ALL_DFLAGS := 
BEIA_ELF := 	
BEIA_PLAIN := 	
BEIA_DIR := 	
BEIA_LINK := 	
BEIA_MFLAGS :=	
BEIA_DFLAGS :=	
BEIA_MVARS :=	
NOT_BEIA_ELF := 	
NOT_BEIA_PLAIN := 	
NOT_BEIA_DIR := 	
NOT_BEIA_LINK := 	
NOT_BEIA_MFLAGS :=	
NOT_BEIA_DFLAGS :=	
NOT_BEIA_MVARS :=	
IATOOLS_ELF := 
IATOOLS_PLAIN := 
IATOOLS_DIR := 	
IATOOLS_LINK := 
IATOOLS_MFLAGS :=
IATOOLS_DFLAGS :=
IATOOLS_MVARS :=
IADEBUG_ELF := 	
IADEBUG_PLAIN := 
IADEBUG_DIR := 	
IADEBUG_LINK := 
IADEBUG_MFLAGS :=
IADEBUG_DFLAGS :=
IADEBUG_MVARS :=
IAINSTALLER_ELF := 	
IAINSTALLER_PLAIN := 
IAINSTALLER_DIR := 	
IAINSTALLER_LINK := 
IAINSTALLER_MFLAGS :=
IAINSTALLER_DFLAGS :=
IAINSTALLER_MVARS :=
IADEVELOP_ELF := 	
IADEVELOP_PLAIN := 
IADEVELOP_DIR := 	
IADEVELOP_LINK := 
IADEVELOP_MFLAGS :=
IADEVELOP_DFLAGS :=
IADEVELOP_MVARS :=
BEOS_ELF := 
BEOS_PLAIN := 	
BEOS_DIR := 	
BEOS_LINK := 	
BEOS_MFLAGS :=	
BEOS_DFLAGS :=	
BEOS_MVARS :=	

